unit HPStack;

interface

uses
  Classes, SysUtils;

type
  THPStack = class
  private
         Mem: double;
         LastX: double;
         Stack: array[0..3] of double;
         Mode: integer;
         Display: integer;
         function FormatString: string;
         procedure ToRad;
         procedure ToDeg;
  public
        procedure SetX(x: double);
        procedure push(x: double);
        function pop: double;
        procedure swap;
        procedure drop;
        procedure dup;
        procedure reset;
        procedure sto;
        procedure rcl;
        procedure memadd;
        procedure memsub;
        function x: string;
        function y: string;
        function z: string;
        function t: string;
        function m: string;
        procedure pi;
        procedure abs;
        procedure int;
        procedure frac;
        procedure sqr;
        procedure sqrt;
        procedure ln;
        procedure log;
        procedure sin;
        procedure cos;
        procedure tan;
        procedure cot;
        procedure arctan;
        procedure arcsin;
        procedure arccos;
        procedure arccot;
        procedure exp;
        procedure exp10;
        procedure add;
        procedure subtract;
        procedure multiply;
        procedure divide;
        procedure pow;
        procedure chgsgn;
        procedure Settings(m, d: integer);
        procedure invert;
        procedure RollUp;
        procedure RollDown;
        procedure SaveLastX;
        procedure RecallLastX;
  end;

implementation

procedure THPStack.ToRad;
begin
     if Mode=0 then Stack[0]:=(System.Pi/180)*Stack[0];
end;

procedure THPStack.ToDeg;
begin
     if Mode=0 then Stack[0]:=(180/System.Pi)*Stack[0];
end;

function THPStack.FormatString: string;
var
   s: string;
begin
     case Display of
          0: s:='%.2f';
          1: s:='%.4f';
          2: s:='%.9g';
          3: s:='%.9e';
     end;
     FormatString:=s;
end;

procedure THPStack.SetX(x: double);
begin
     Stack[0]:=x;
end;

procedure THPStack.push(x: double);
var
   i: integer;
begin
     for i:=3 downto 1 do Stack[i]:=Stack[i-1];
     Stack[0]:=x;
end;

function THPStack.pop: double;
var
   i: integer;
   x: double;
begin
     x:=Stack[0];
     for i:=0 to 2 do Stack[i]:=Stack[i+1];
     pop:=x;
end;

procedure THPStack.swap;
var
   x: double;
begin
     x:=Stack[0];
     Stack[0]:=Stack[1];
     Stack[1]:=x;
end;

procedure THPStack.drop;
var
   i: integer;
begin
     for i:=0 to 2 do Stack[i]:=Stack[i+1];
end;

procedure THPStack.dup;
var
   i: integer;
begin
     for i:=3 downto 1 do Stack[i]:=Stack[i-1];
end;

procedure THPStack.reset;
var
   i: integer;
begin
     for i:=0 to 3 do Stack[i]:=0;
     Mem:=0;
     LastX:=0;
end;

procedure THPStack.sto;
begin
     Mem:=Stack[0];
end;

procedure THPStack.rcl;
begin
     push(Mem);
end;

procedure THPStack.memadd;
begin
     Mem:=Mem+Stack[0];
end;

procedure THPStack.memsub;
begin
     Mem:=Mem-Stack[0];
end;

function THPStack.x: string;
begin
     x:=format(FormatString, [Stack[0]]);
end;

function THPStack.y: string;
begin
     y:=format(FormatString, [Stack[1]]);
end;

function THPStack.z: string;
begin
     z:=format(FormatString, [Stack[2]]);
end;

function THPStack.t: string;
begin
     t:=format(FormatString, [Stack[3]]);
end;

function THPStack.m: string;
begin
     m:=format(FormatString, [Mem]);
end;

procedure THPStack.pi;
begin
     push(System.Pi);
end;

procedure THPStack.abs;
begin
     Stack[0]:=System.Abs(Stack[0]);
end;

procedure THPStack.int;
begin
     Stack[0]:=System.Int(Stack[0]);
end;

procedure THPStack.frac;
begin
     Stack[0]:=System.Frac(Stack[0]);
end;

procedure THPStack.sqr;
begin
     Stack[0]:=System.Sqr(Stack[0]);
end;

procedure THPStack.sqrt;
begin
     Stack[0]:=System.Sqrt(Stack[0]);
end;

procedure THPStack.ln;
begin
     Stack[0]:=System.Ln(Stack[0]);
end;

procedure THPStack.log;
begin
     Stack[0]:=System.Ln(Stack[0])/System.Ln(10);
end;

procedure THPStack.sin;
begin
     ToRad;
     Stack[0]:=System.Sin(Stack[0]);
end;

procedure THPStack.cos;
begin
     ToRad;
     Stack[0]:=System.Cos(Stack[0]);
end;

procedure THPStack.tan;
begin
     ToRad;
     Stack[0]:=System.Sin(Stack[0])/System.Cos(Stack[0]);
end;

procedure THPStack.cot;
begin
     ToRad;
     Stack[0]:=System.Cos(Stack[0])/System.Sin(Stack[0]);
end;

procedure THPStack.arctan;
begin
     Stack[0]:=System.ArcTan(Stack[0]);
     ToDeg;
end;

procedure THPStack.arcsin;
begin
     Stack[0]:=System.ArcTan(Stack[0]/System.Sqrt(1-System.Sqr(Stack[0])));
     ToDeg;
end;

procedure THPStack.arccos;
begin
     Stack[0]:=System.ArcTan(Stack[0]/System.Sqrt(1-System.Sqr(Stack[0])));
     push(System.Pi/2);
     swap;
     subtract;
     ToDeg;
end;

procedure THPStack.arccot;
begin
     Stack[0]:=System.ArcTan(Stack[0]);
     push(System.Pi/2);
     swap;
     subtract;
     ToDeg;
end;

procedure THPStack.exp;
begin
     Stack[0]:=System.Exp(Stack[0]);
end;

procedure THPStack.exp10;
begin
     Stack[0]:=System.Exp(Stack[0]*System.Ln(10));
end;

procedure THPStack.add;
var
   i: integer;
begin
     Stack[0]:=Stack[0]+Stack[1];
     for i:=1 to 2 do Stack[i]:=Stack[i+1];
end;

procedure THPStack.subtract;
var
   i: integer;
begin
     Stack[0]:=Stack[1]-Stack[0];
     for i:=1 to 2 do Stack[i]:=Stack[i+1];
end;

procedure THPStack.multiply;
var
   i: integer;
begin
     Stack[0]:=Stack[0]*Stack[1];
     for i:=1 to 2 do Stack[i]:=Stack[i+1];
end;

procedure THPStack.divide;
var
   i: integer;
begin
     Stack[0]:=Stack[1]/Stack[0];
     for i:=1 to 2 do Stack[i]:=Stack[i+1];
end;

procedure THPStack.pow;
var
   i: integer;
begin
     Stack[0]:=System.Exp(System.Ln(Stack[1])*Stack[0]);
     for i:=1 to 2 do Stack[i]:=Stack[i+1];
end;

procedure THPStack.chgsgn;
begin
     Stack[0]:=-1*Stack[0];
end;

procedure THPStack.Settings(m, d: integer);
begin
     Mode:=m;
     Display:=d;
end;

procedure THPStack.invert;
begin
     Stack[0]:=1/Stack[0];
end;

procedure THPStack.RollUp;
var
   x: double;
begin
     x:=Stack[3];
     dup;
     Stack[0]:=x;
end;

procedure THPStack.RollDown;
var
   x: double;
begin
     x:=Stack[0];
     drop;
     Stack[3]:=x;
end;

procedure THPStack.SaveLastX;
begin
     LastX:=Stack[0];
end;

procedure THPStack.RecallLastx;
begin
     Stack[0]:=LastX;
end;

end.
